_ENV = _G.PartOrdering

-- compare mods isn't used anymore
-- We either compare parts, or part+mod combinations
-- the utility of comparing mods without the context of a part seems limited 
-- and by dropping it I hope to keep the complexity manageable

function greedy(gen_part_ordering)
  reset_all_empire_part_info()

  return function(args)
    local design = Design.new(args)
    local empire,config_table,loadout_hint in args
    local part_info =  empire : available_part_info()
    local specs = {}
    vprint('debug',design:debug_string())
    local mount_mod_conflicts in Design!
    local _TRACE_INFO=empire.name

    -- we only ever add (or replace) a new part for a mount, 
    -- or we add a new mod to an existing mount.

    local function find_best_addition(compare_fun)
      local best = {}

      for i,mount,name in design.mounts()

        SCOPE_call('current_mount_i',i,|) function() --necessary for mixed_goals() to work

        local candidates = {}
        for part_name, part in rpairs | part_info
          if not invalid_part_change(mount,part)
            candidates[#candidates+1] = { ..part_name, ..part }

            for mod in rpairs | part.unlocked_mods?
              local cant = invalid_mod_add(mount,part,mod)
              if not cant or (part_name == name and cant == 'mod_already_added')
                candidates[#candidates+1] = { ..part_name, ..part, ..mod }
              end
            end

          end
        end

        local _,this = get_best_value(candidates,|) function (part1, part2)
          return not compare_fun(mount, part1.part, part1.mod, mount, part2.part, part2.mod)
        end

        if this
          local part_name, part, mod in this

          --print(i, 'best candidate!', part.part, mod)
          if name ~= part_name

            if not best.part or compare_fun(mount, part, mod, best.mount, best.part, best.mod )
              cleartable(best)
              insert(best, ..i, ..mount, ..part)
            end

          else
            --looks like we already have the best part installed. Consider adding mods
            for mod in rpairs | part.unlocked_mods?
              if not invalid_mod_add(mount,part,mod)
                if not best.part or compare_fun( mount, part, mod, best.mount, best.part, best.mod )
                  cleartable(best)
                  insert( best, ..i, ..mount , ..part, ..mod)
                end
              end
            end

          end

        end end

      end

      --vprint('returning',v2s | best)

      return best
    end

    -- a generic temp table that can be used by the autodesign
    -- utilities however they see fit.
    --
    -- (this may be an ill-advised kludge)
    local temp_autodesign_flags = {}


    local new_design = SCOPE_call(..temp_autodesign_flags, ..design,..specs, ..part_info, |) function()

      local _TRACE_INFO=design.design_name
      local part_ordering = gen_part_ordering()

      local compare_fun,on_placed in part_ordering

      -- auto-include a top level design-hint ordering function,
      -- but only if it looks like this design has an applicable hint field.
      if args.design_hint
        for i,mount in design.mounts()
          if mount.hints?[args.design_hint]
            vprint('prepending design hint',args.design_hint)
            compare_fun in PartOrdering.lexical {'design_hints', part_ordering}
            break 
          end
        end
      end

      do
        -- start by adding in the best possible reactors in all applicable slots
        for i,mount in design.mounts()
          design.update_hull_specs(specs)
          local best_reactor,best_power = get_max_key(part_info,|) function(part_name,part)
            --print(..part_name,..part)
            if invalid_part_change(mount,part)
              return -1
            end
            return part.increases?.power?(v,mount,_SAFE)
          end

          if best_power > 0
            design.filled_mounts[i] = best_reactor
          end
        end

      end

      --vprint('staring w/',design.debug_string())

      compare_fun = timed_function( 'top_compare', compare_fun) 

      for try=1,200
        design.update_hull_specs(specs)
        temp_autodesign_flags.insufficient_power=nil


        local mod,part,i in find_best_addition( compare_fun )
        print("best addition for", i, part?.part, mod)

        if part
          design.filled_mounts[i] = part!.part
          if mod
            design.active_mods[i] = design.active_mods[i] or {}
            design.active_mods[i][mod] = true
          end
        else
          --vprint("looks like we didn't find anything")
          return design
        end

        design.update_hull_specs(specs)

        if (try > 1) and (specs.power < 0)
          errorf('strange power caculation after adding %s',part.part)
        end
        vprint('placed',i,part.part,mod)
        on_placed(i,part,mod)
      end
    end

    if new_design
      return new_design
    end

    local _TRACE_INFO = design.design_name
    error 'autodesign process didnt terminate!'

    return design
  end
end

invalid_part_change = timed_function("part invalid",|) function (mount, part_info )

  if not mount_allowed_types[mount.mtype][part_info.type]
    return 'incompatible_part_type'
  end

  local design,temp_autodesign_flags,specs in SCOPE
  local active_mods in design
  local current_power_use = 0
  local i in mount

  local old_part_info = SCOPE.part_info[ design.filled_mounts[i] ]
  if old_part_info
    --print("power use", old_part_info.part, old_part_info:required_power(mount, active_mods[i]?), old_part_info.increases?.power?(old_part_info,mount) or 0)
    current_power_use = old_part_info : required_power( mount, active_mods[i]? )
    current_power_use  -=  old_part_info.increases?.power?(old_part_info,mount) or 0
  end

  local required_power = part_info : required_power( mount, _SAFE )
  required_power -= part_info.increases?.power?(part_info,mount) or 0

  if floor(specs.power) < ceil( required_power - current_power_use ) --these should be integers already...
    if not  temp_autodesign_flags.insufficient_power
      --print('insuffient power for',part_info.part,mount.i)
      temp_autodesign_flags.insufficient_power=true
    end

    return 'insufficient_power'
  end
end

function complete_ordering(ordering)
  if ordering.compare_fun
    return ordering
  end

  local compare_parts in ordering!
  
  ordering.compare_fun = ordering.compare_fun or function(mount1, part1, mod1, mount2, part2, mod2)
    if not (part1 and part2)
      if part1
        return true
      end
      if part2
        return false
      end
      return
    end

    return compare_parts(mount1,part1,mount2,part2)
  end

  if ordering.timing_name
    ordering.compare_fun = timed_function(ordering.timing_name,ordering.compare_fun)
  end

  return ordering
end

local function default_mount_comp(mount1,mount2)
  -- always favor filling in new mounts over mounts that already have content.
  local design in SCOPE
  if design.filled_mounts[mount1.i] and not design.filled_mounts[mount2.i]
    return false
  end
  if design.filled_mounts[mount2.i] and not design.filled_mounts[mount1.i]
    return true
  end

  if mount1.i < mount2.i return true end
  if mount1.i > mount2.i return false end
end

function hardpoint_specific(hardpoint_to_comp, mount_comp) 

  mount_comp = mount_comp or default_mount_comp

  local function compare_fun(mount1, part1, mod1, mount2,part2, mod2)

    if not (part1 and part2)
      if part1
        return true
      end
      if part2
        return false
      end
      return
    end

    -- comparing across mounts...  
    local comp = mount_comp(mount1,mount2)
    if comp ~= nil
      return comp 
    end

    -- ok, at this point, we should be comparing two different parts
    -- proposed for the same mount

    -- because mount_comp() can be overriden, it may not be true that mount1==mount2
    -- but they should be equivalent (or else mount_comp is breaking it's contract)

    local ordering = get_orderfun | hardpoint_to_comp(mount1)
    local compare_fun in ordering


    if compare_fun( mount1, part1,mod1, mount2,part2, mod2) 
      return true 
    end

    if compare_fun( mount2, part2,mod2, mount1,part1, mod1 ) 
      return false 
    end
  end


  local function on_placed(i, part, mod)
    local mount = SCOPE.design.config_table.available_mounts![i]
    hardpoint_to_comp(mount).on_placed?(i, part, mod)
  end


  return complete_ordering {
    ..compare_fun,
    ..on_placed,
    ..hardpoint_to_comp,
    typename = 'hardpoint_specific',
  }
end


do
  local shorthands = {
    turrets = set('SmallTurret', 'Turret'), --now refers specifically to light and medium turrets
    heavy_turrets = set('LargeTurret', 'HugeTurret'),
    missiles = set('Missile','Torpedo'),
    heavy = set('LargeTurret', 'HugeTurret', 'LargeFixed', 'MaskedBeamMounts','Siege'),
  }

  function PartOrdering.new_hardpoint_comp_fun(args, default_fun)

    assert(args.default or default_fun)

    --local rval = copy | args

    return function(mount)
      local hardpoint_type, mtype in mount

      local fun = args[hardpoint_type] or args[mtype]
      if fun
        return fun
      end

      for shorthand, alias_set ; shorthands
        local fun = args[shorthand]
        if fun and (alias_set[hardpoint_type] or alias_set[mtype])
          return fun
        end
      end

      if default_fun
        return default_fun(mount)
      end
      return args.default
    end
  end

end


--Similar to compare_parts_using_metric(), but compares parts and mods
function compare_fun_using_metric(metric)
  return function( mount1, part1, mod1, mount2, part2, mod2 )
    return metric(part1,mount1,mod1) > metric(part2,mount2,mod2)
  end
end


function from_compare_metric_only(fun)
  return function()
    return complete_ordering {
      compare_fun = compare_fun_using_metric(fun),
      typename = 'by_metric',
    }
  end
end

--cycle through each item in compare_list, every time we place a part
function PartOrdering.mixed_goals(compare_list)
  local placed_count = 0

  local slot_to_comp = {}

  --need to create each orderfun in compare_list exactly once, or else nested mixed goals will not work.
  local orderfuns = {}
  for i,v in ipairs(compare_list) do
    orderfuns[i] = get_orderfun | v
  end

  local current = orderfuns[1]
  
  local function compare_fun(mount1,part1,mod1,mount2,part2,mod2)
    local ordering = (slot_to_comp[SCOPE.current_mount_i] or current)
    return ordering.compare_fun( mount1, part1,mod1, mount2, part2, mod2 )
  end

  local function on_placed(i, part, mod)
    current.on_placed?(i, part, mod)

    if not slot_to_comp[i]
      slot_to_comp[i] = current

      placed_count++

      local o = 1 + (placed_count % #compare_list)
      current = orderfuns[o]
    end

  end

  return {
    ..compare_fun,
    ..on_placed,
    typename = 'mixed_goals' 
  }
end

----- Ordering specific parts

-- requests at least n of a specific part
function request_specific_part(desired_part, n)
  n = n or 1

  return from_part_metric_only | function(part,mount)
    local count = 0
    local already_placed = false

    for i,m,name in SCOPE.design.mounts()
      if name == desired_part
        count++

        if i==mount.i
          already_placed = true
        end
      end
    end

    if part.part == desired_part 
      -- if the desired part is already in the mount we are considering adding it to,
      -- keep it there as long as we aren't exceeding our quota
      if already_placed and count <= n
        return 1
      end

      if count < n
        return 1 -- add a new copy of the desired part
      end
    end

    return 0
  end
end

PartOrdering.cloaking = request_specific_part('Distortion Generator', 1)

PartOrdering.autorepair_1x = request_specific_part('Autorepair System', 1)

PartOrdering.extended_range = request_specific_part('Warp Lane Amplifier', 1)

----- These can now be used to order various weapon mods

PartOrdering.high_damage = from_compare_metric_only | function(part,mount,mod)
  if not part.is_weapon
    return -1
  end

  if part.specials?['Shield Damage Only']
    return -1
  end

  local damage = -1

  --fix PartOrdering.high_damage not checking fighter weapons
  if part.is_fighter
    local ftr_weapon,alt_weapon = get_fighter_weapon(part.part, SCOPE.design.empire)
    if alt_weapon
      damage = max(WEAPONS[ftr_weapon].damage_rank, WEAPONS[alt_weapon].damage_rank)
    else
      damage = WEAPONS[ftr_weapon].damage_rank
    end
  else
    damage = part.damage_rank
  end

  if part.is_launcher 
    damage *= (part.submunitions or 1)
  end

  if mod == 'Rapid Fire'
    damage *= 2
  end
  if mod == 'Armor Piercing'
    damage *= 1.35
  end
  if mod == 'Shield Piercing'
    damage *= 1.2
  end

  return mount.n * damage
end


  --!! PartOrdering.long_range
PartOrdering.long_range = from_compare_metric_only | function(part,mount,mod)
  if not part.is_weapon
    return 0
  end

  local v=0
  if part.is_unlimited_range
    v = 4
  elseif part.type=='siege'
    v = 3
  elseif part.is_heavy
    v = 2
  elseif not part.is_pd or part.is_anti_missile --anti missiles have longer range than other PD
    v = 1
  elseif part.is_pd
    v = 0.5
  end

  if mod == 'Long Range'
    v += 0.5
  end
  if mod == 'Accurate' or mod == 'Focusing Lens'
    v += 0.25
  end

  return v 
end

  --!! PartOrdering.pd
PartOrdering.pd = from_part_metric_only | function(part, mount)
  if not part.is_weapon or part.cant_attack_trackers
    return 0
  end

  local v = 0
  if part.is_pd
    v = 2
  else
    v = 1
  end

  if mod == 'Accurate' --very useful for PD
    v += 0.7
  end
  if mod == 'Rapid Fire' --extra shots helps to land more hits
    v += 0.35
  end

  --increased PD coverage can be useful in fleet situations
  if mod == 'Long Range' 
    v += 0.3
  end
  if mod == 'Focusing Lens'
    v += 0.1
  end

  return v
end

PartOrdering.can_attack_trackers = from_part_metric_only | function(part, mount)
  return part.cant_attack_trackers and 0 or 1
end

PartOrdering.heavy_weapon = from_part_metric_only | function(part, mount)
  return part.is_heavy and 1 or 0
end

function PartOrdering.big_guns()
  return lexical { 'direct', 'heavy_weapon', 'high_damage', 'long_range' }
end

PartOrdering.direct = from_part_metric_only | function(part, mount)
  return part.is_direct and 1 or 0
end

PartOrdering.cost = from_part_metric_only | function(part,mount)
  local hammer_cost, metal_cost = part : costs(mount,_SAFE)
  return -(hammer_cost+2*metal_cost) --lower cost is better!
end

function PartOrdering.pure_pd()
  -- mix short-range PD and anti-missiles
  return mixed_goals {
    lexical { 'direct', 'pd', 'high_damage' },
    lexical { 'direct', 'pd', 'long_range' },
  }
end

function PartOrdering.some_direct()
  return mixed_goals {
    lexical {'direct','high_damage','pd'}, -- choose the direct weapon first
    lexical {'high_damage','long_range'},
  }
end

function PartOrdering.direct_weapons()
  return lexical { 'direct','high_damage','pd' }
end

-- For well rounded, balanced designs. Favours direct-fire weapons a more than in vanilla,
-- since the AI should now have dedicated missile versions of designs it can use.
function PartOrdering.generic_autodesign()
  return hardpoint_specific | new_hardpoint_comp_fun {
    default = lexical { 'direct_weapons', 'cost', timing_name2='default generic lexical' },
    --ignore the 'direct' pref for missile slots (since anti-missile are direct, the default causes problems)
    missiles = lexical {'high_damage','long_range', 'cost', timing_name2='missiles lexical'},
    Hangar = lexical { mixed_goals { 'high_damage', 'pure_pd' }, 'cost', timing_name2='hangar lexical' }, --mix bombers and interceptors
    System = lexical { 'shields', 'armor', 'extended_range', 'speed', 'cost', timing_name2='system lexical' },
    Armor = 'armor',
    Station = lexical { 'construction', 'shields', 'armor', 'cost', },
    turrets = lexical { mixed_goals { 'pure_pd', 'direct_weapons' }, 'cost', timing_name2='turret lexical' },
  } 
end

-- Point defence
function PartOrdering.escort_autodesign()
  return hardpoint_specific | new_hardpoint_comp_fun {
    default = lexical { 'pure_pd', 'cost', timing_name2='default generic lexical' },
    --we don't put PD in the heavy mounts... but we do make sure that what we put there can attack trackers
    heavy = lexical { 'can_attack_trackers', 'high_damage', 'long_range', 'cost', timing_name2='heavy lexical' },
    System = lexical { 'shields', 'armor', 'extended_range', 'speed', 'cost', timing_name2='system lexical' },
    Armor = 'armor',
    Station = lexical { 'construction', 'shields', 'armor', 'cost', },
    turrets = lexical {  'pure_pd', 'cost', timing_name2='turret lexical' },
  } 
end

-- Siege. Long range stand-off capability - i.e. missiles, with some PD.
function PartOrdering.siege_autodesign()
  return hardpoint_specific | new_hardpoint_comp_fun {
    default = lexical { 'long_range', 'high_damage', 'cost', timing_name2='default generic lexical' },
    System = lexical { 'shields', 'armor', 'extended_range', 'speed', 'cost', timing_name2='system lexical' },
    Armor = 'armor',
    Station = lexical { 'construction', 'shields', 'armor', 'cost', },
    -- even siege should have some point defence
    turrets = lexical { mixed_goals { 'pure_pd', lexical { 'long_range', 'high_damage' } }, 'cost', timing_name2='turret lexical' },
  } 
end

-- Somewhere between siege and balanced. We try to put long range direct-fire weapons in the heavy mounts, 
-- so it is less missile heavy than Siege but still retaining a long range focus.
function PartOrdering.big_guns_autodesign()
  return hardpoint_specific | new_hardpoint_comp_fun {
    default = lexical { 'direct_weapons', 'cost', timing_name2='default generic lexical' },
    --ignore the 'direct' pref for missile slots (since anti-missile are direct, the default causes problems)
    missiles = lexical {'high_damage','long_range', 'cost', timing_name2='missiles lexical'},
    heavy = lexical { 'big_guns', 'cost', timing_name2='heavy lexical' },
    System = lexical { 'shields', 'armor', 'extended_range', 'speed', 'cost', timing_name2='system lexical' },
    Armor = 'armor',
    Station = lexical { 'construction', 'shields', 'armor', 'cost', },
    turrets = lexical { mixed_goals { 'pure_pd', 'direct_weapons' }, 'cost', timing_name2='turret lexical' },
  } 
end

function patch_hardpoint_comp_fun(orderfun, hardpoint_override, mount_comp)
  local hardpoint_to_comp in get_orderfun | orderfun

  return function()
    local hardpoint_to_comp = new_hardpoint_comp_fun(hardpoint_override, hardpoint_to_comp)
    return hardpoint_specific(hardpoint_to_comp, mount_comp)
  end
end

-- For Gremak
function new_raider_ordering(name)
  local mount_comp = function(mount1,mount2)
    local design in SCOPE

    -- if we don't yet have a cloaking device, then our first
    -- priority is to choose a part for a mount that we can put a
    -- cloaking device in.
    --
    -- (this needs to prempt the empty>non_empty ordering rule,
    -- because all designs are initially filled with nothing but
    -- reactors.)
    if not SCOPE.design.has_cloaking_part()
      if (mount1.mtype=='System') and  (mount2.mtype~='System')  return true end
      if (mount2.mtype=='System') and  (mount1.mtype~='System')  return true end
    end

    -- fall back on default behaviors.
    return default_mount_comp(mount1,mount2)
  end

  local orderfun = get_orderfun | name
  local hardpoint_to_comp in orderfun

  local cloaking_prefs = {
    System = lexical { 'cloaking', 'shields', 'armor', 'cost', timing_name2='system lexical' },
  }

  PartOrdering[name .. '_raider'] = patch_hardpoint_comp_fun(name, cloaking_prefs, mount_comp)
end


do
  local generate_raider_prefs = {
    'generic_autodesign',
    'escort_autodesign',
    'siege_autodesign',
    'big_guns_autodesign',
  }
  for i, name ; generate_raider_prefs
    new_raider_ordering(name)
  end
end



if lua_load_complete
  for i,empire ; all_empires 
    if empire == gui_player()
      redesign_ships(empire)
      for key,design ; empire.designs
        --print(key, design.design_name, design.config_name, design.design_hint)
      end
    end 
  end
end  