

_G.loadout_hints_to_prefs = {
  balanced   = 'generic_autodesign',
  escort     = 'escort_autodesign',
  standoff   = 'siege_autodesign',
  heavy_guns = 'big_guns_autodesign',
}

--in an ideal world maybe these would be attached to the configs themselves
_G.loadout_hints = {}

local function config_loadout(config_name)
  return function(loadouts)
    return catch(config_name, set(unpack(loadouts)))
  end
end

local function theme_loadouts(theme)
  return function(t)
    loadout_hints[theme] = {}

    for i,v ; t
      local config_name, loadout = v()
      loadout_hints[theme][config_name] = loadout
    end
    
  end
end

theme_loadouts 'Hawk' {
  config_loadout 'Destroyer' { 'balanced', 'standoff' },
  config_loadout 'Light Cruiser' { 'balanced', 'standoff' },
  config_loadout 'Missile Cruiser' { 'standoff' },

  --with 4 turret mounts, all three configs will have at least 2 of 4 be PD weapons
  config_loadout 'Escort Cruiser' { 'balanced', 'escort', 'standoff'},
  config_loadout 'Military Transport' { 'balanced', 'escort' },

  config_loadout 'Heavy Cruiser' { 'balanced', 'heavy_guns', 'standoff' },
  config_loadout 'Battlecruiser' { 'heavy_guns', 'standoff' },

  --for carriers, escort is all interceptors, siege is all bombers, balanced is a mix
  config_loadout 'Escort Carrier' { 'escort', 'standoff' },
  config_loadout 'Light Carrier' { 'balanced', 'escort', 'standoff' }, 
  config_loadout 'Fleet Carrier' { 'balanced' },
  
  config_loadout 'Fleet Base' { 'standoff' },
  config_loadout 'Star Base' { 'standoff' },
}

theme_loadouts 'Star' {
  config_loadout 'Destroyer' { 'balanced', 'escort' },
  config_loadout 'Torpedo Destroyer' { 'standoff' },
  config_loadout 'Armed Merchant' { 'escort' },
  config_loadout 'Military Transport' { 'balanced', 'escort' },
  config_loadout 'Cruiser' { 'balanced', 'heavy_guns', 'standoff' },

  config_loadout 'Carrier' { 'balanced', 'standoff' },
  config_loadout 'Heavy Carrier' { 'balanced' }, --is this used?

}

theme_loadouts 'Blade' {
  config_loadout 'Destroyer' { 'balanced' },
  config_loadout 'Light Cruiser' { 'balanced', 'standoff' },
  config_loadout 'Escort Cruiser' { 'escort' },
  config_loadout 'Command Cruiser' { 'balanced' },

  config_loadout 'Cruiser' { 'balanced', 'heavy_guns', 'standoff' },

  --'heavy_guns' replaces 'balanced' on the slower ships, since they need the range
  config_loadout 'Battlecruiser' { 'heavy_guns', 'standoff' },
  config_loadout 'Battleship' { 'heavy_guns', 'standoff' },

  config_loadout 'Assault Carrier' { 'balanced' },
}

theme_loadouts 'Box' {
  config_loadout 'Frigate' { 'balanced' },
  config_loadout 'Destroyer' { 'balanced', 'standoff' },
  config_loadout 'Advanced Destroyer' { 'balanced', 'standoff' },
  config_loadout 'Heavy Destroyer' { 'balanced', 'heavy_guns' },
  config_loadout 'Torpedo Destroyer' { 'standoff' },
  config_loadout 'Light Cruiser' { 'balanced', 'standoff' },

  --only two turret mounts, and a fixed mount. So anything other than escort will be PD-poor
  config_loadout 'Escort Cruiser' { 'escort' },

  config_loadout 'Heavy Cruiser' { 'balanced', 'heavy_guns', 'standoff'  },
  config_loadout 'Battleship' { 'heavy_guns', 'standoff' },

  config_loadout 'Carrier' { 'balanced' },
}

theme_loadouts 'Hammer' {
  config_loadout 'Destroyer' { 'balanced', 'escort', 'standoff' },
  config_loadout 'Light Cruiser' { 'balanced', 'standoff' },
  config_loadout 'Missile Cruiser' { 'standoff' },
  config_loadout 'Assault Cruiser' { 'balanced', 'heavy_guns' },

  config_loadout 'Heavy Cruiser' { 'balanced', 'heavy_guns', 'standoff' },
  config_loadout 'Battleship' { 'heavy_guns', 'standoff'  },

  config_loadout 'Carrier' { 'balanced' },
}

theme_loadouts 'Hat' {
  config_loadout 'Destroyer' { 'balanced', 'escort', 'standoff' },
  config_loadout 'Missile Destroyer' { 'standoff' },
  config_loadout 'Light Cruiser' { 'balanced', 'standoff', 'heavy_guns' },
  config_loadout 'Missile Cruiser' { 'standoff' },

  config_loadout 'Heavy Cruiser' { 'balanced', 'heavy_guns', 'standoff' },
  config_loadout 'Juggernaut' { 'heavy_guns', 'standoff' },

  config_loadout 'Carrier' { 'balanced' },
}

theme_loadouts 'Manta' {
  config_loadout 'Destroyer' { 'balanced', 'escort', 'standoff' },
  config_loadout 'Escort Cruiser' { 'balanced', 'escort', 'standoff'}, -- 2 very large turret mounts, so 1 of 2 being PD is fine
  config_loadout 'Military Transport' { 'balanced', 'escort' },

  config_loadout 'Gunship' { 'heavy_guns' },

  config_loadout 'Heavy Cruiser' { 'balanced', 'heavy_guns', 'standoff' },
  config_loadout 'Battlecruiser' { 'heavy_guns', 'standoff' },
  config_loadout 'Dreadnought' { 'heavy_guns', 'standoff' },

  config_loadout 'Carrier' { 'balanced' },
}